// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// Test harness for the breadth first search traversal code
// Use any of the files in this directory with a .gph extension.

#include "book.h"
#include "grlist.h"
#include "grmat.h"
#include "aqueue.h"

// Queue used by breadth-first traversal
Queue<int> *Q;


// Start with some implementations for the abstract functions
void PreVisit(Graph *G, int v)
{
    cout << "PreVisit vertex " << v << "\n";
}


void PostVisit(Graph *G, int v)
{
    cout << "PostVisit vertex " << v << "\n";
}


void BFS(Graph *G, int start, Queue<int> *Q)
{
    int v, w;
    Q->enqueue(start);         // Initialize Q
    G->setMark(start, VISITED);

    // Process all vertices on Q
    while(Q->length() != 0)
    {
        v = Q->dequeue();
        PreVisit(G, v);          // Take appropriate action

        for(w = G->first(v); w < G->n(); w = G->next(v, w))
        {
            if(G->getMark(w) == UNVISITED)
            {
                G->setMark(w, VISITED);
                Q->enqueue(w);
            }
        }
    }
}


void doTraverse(Graph *G, int v)
{
    cout << "\tdoTraverse vertex " << v << "\n" << endl;
    BFS(G, v, Q);
}


void graphTraverse(Graph *G)
{
    int v;

    for(v = 0; v < G->n(); v++)
        G->setMark(v, UNVISITED);  // Initialize mark bits

    for(v = 0; v < G->n(); v++)
    {
        if(G->getMark(v) == UNVISITED)
        {
            cout << "The graph is connected/reachable if this prints once." << endl;
            doTraverse(G, v);
        }
    }
}


// Test Breadth First Search:
int main()
{
    Graph *G;

    FILE *fid;
    fid = fopen("bkexamp.gph", "rt");

    G = createGraph<Graphl>(fid);
    // if matrix implementation used
    //G = createGraph<Graphm>(fid);

    Q = new AQueue<int>(G->n());
    Gprint(G);
    graphTraverse(G);

    return 0;
}
