// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// Test harness for the depth first search traversal code
// Use any of the files in this directory with a .gph extension.

#include "book.h"
#include "grlist.h"
#include "grmat.h"


void PreVisit(Graph *G, int v)
{
    cout << "PreVisit vertex " << v << "\n";
}


void PostVisit(Graph *G, int v)
{
    cout << "PostVisit vertex " << v << "\n";
}


// Depth first search
void DFS(Graph *G, int v)
{
    PreVisit(G, v);           // Take appropriate action
    G->setMark(v, VISITED);

    for(int w = G->first(v); w < G->n(); w = G->next(v, w))
    {
        if(G->getMark(w) == UNVISITED)
            DFS(G, w);
    }

    PostVisit(G, v);          // Take appropriate action
}


void doTraverse(Graph *G, int v)
{
    cout << "\tdoTraverse vertex " << v << "\n" << endl;
    DFS(G, v);
}


void graphTraverse(Graph *G)
{
    int v;

    for(v = 0; v < G->n(); v++)
        G->setMark(v, UNVISITED);  // Initialize mark bits

    for(v = 0; v < G->n(); v++)
    {
        if(G->getMark(v) == UNVISITED)
        {
            cout << "The graph is connected/reachable if this prints once." << endl;
            doTraverse(G, v);
        }
    }
}


// Test Depth First Search:
int main()
{
    Graph *G;

    FILE *fid;
    fid = fopen("bkexamp.gph", "rt");

    G = createGraph<Graphm>(fid);
    // if matrix implementation used
    //G = createGraph<Graphm>(fid);

    Gprint(G);
    graphTraverse(G);

    return 0;
}

