// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// Test program for Prim's MCST algroithm. This version uses
// a priority queue to find the next closest vertex,
// similar to the second version of Dijkstra's algorithm
// Use any of the files in this directory with a .gph extension.

#include "book.h"
#include "grlist.h"
#include "grmat.h"
#include "heap.h"


void AddEdgetoMST(int v1, int v2, int w)
{
    cout << "edge " << v1 << " to " << v2 << " with weight: " << w << endl;
}

// Simple class to represent objects to be stored in the priority queue
// Store a vertex and its best known distance
class DijkElem
{
    public:
        int vertex, distance;

        DijkElem()
        {
            vertex = -1;
            distance = -1;
        }

        DijkElem(int v, int d)
        {
            vertex = v;
            distance = d;
        }
};


// Comparator class for comparing DijkElem's
class DDComp
{
    public:
        static bool prior(DijkElem x, DijkElem y)
        {
            return x.distance < y.distance;
        }
};

// Prim's MST algorithm: priority queue version
void Prim(Graph *G, int *D, int s)
{
    int i, v, w;            // v is current vertex

    D[s] = 0; // set current nodes distance to 0

    int V[G->n()];         // V[I] will store I's closest neighbor

    DijkElem temp;
    temp.distance = 0;
    temp.vertex = s;

    DijkElem E[G->e()];     // array with lots of space (for heap)
    E[0] = temp;            // Initialize heap array

    heap<DijkElem, DDComp> H(E, 1, G->e()); // Create heap

    for(i = 0; i < G->n(); i++)
    {
        do
        {
            if(H.size() == 0)
            {
                return; // Nothing to remove
            }

            temp = H.removefirst();
            v = temp.vertex;
        }
        while(G->getMark(v) == VISITED);

        G->setMark(v, VISITED);

        if(v != s)
            AddEdgetoMST(V[v], v, G->weight(V[v], v)); // Add edge to MST

        if(D[v] == INFINITY)
        {
            return;    // Unreachable vertices
        }

        for(w = G->first(v); w < G->n(); w = G->next(v, w))
        {
            if(G->weight(v, w) < D[w])
            {
                D[w] = G->weight(v, w);
                V[w] = v;        // Update who it came from
                temp.distance = D[w];
                temp.vertex = w;

                // Insert new distance in heap
                H.insert(temp);
            }
        }
    }
}


// Prim's MCST algorithm: use priority queue
// Version for Adjancency Matrix representation
int main()
{
    Graph *G;
    FILE *fid;
    fid = fopen("bkexamp.gph", "rt");

    G = createGraph<Graphm>(fid);
    // For list backend
    // G = createGraph<Graphl>(fid);

    Gprint(G);

    int D[G->n()];

    // Initialize
    for(int i = 0; i < G->n(); i++)
    {
        D[i] = INFINITY;
    }

    cout << "This is your MST: " << endl;
    Prim(G, D, 0);

    cout << "\nD array:\n";

    for(int k = 0; k < G->n(); k++)
    {
        cout << D[k] << " ";
    }

    cout << endl;

    return 0;
}
