// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// Test program for Topological Sort using DFS
// Use any of the files in this directory with a .gph extension.

#include "book.h"
#include "grlist.h"
#include "grmat.h"

void printout(int v)
{
    cout << v << " ";
}

void tophelp(Graph *, int);


// Topological sort: recursive
void topsort(Graph *G)
{
    int i;

    for(i = 0; i < G->n(); i++)      // Initialize Mark array
    {
        G->setMark(i, UNVISITED);
    }

    for(i = 0; i < G->n(); i++)      // Process all vertices
    {
        if(G->getMark(i) == UNVISITED)
        {
            cout << "This prints once if graph is connected/reachable." << endl;
            tophelp(G, i);            // Call recursive helper function
        }
    }
}


// Process vertex v depth first
void tophelp(Graph *G, int v)
{
    G->setMark(v, VISITED);

    for(int w = G->first(v); w < G->n(); w = G->next(v, w))
    {
        if(G->getMark(w) == UNVISITED)
        {
            tophelp(G, w);
        }
    }

    printout(v);                  // PostVisit for Vertex v
}


// Test Topological sort using DFS
int main()
{
    Graph *G;
    FILE *fid;
    fid = fopen("bktsort.gph", "rt");

    G = createGraph<Graphl>(fid);
    // if matrix implementation used
    //G = createGraph<Graphm>(fid);

    Gprint(G);

    cout << endl << "Topological sort is: " << endl;
    topsort(G);
    cout << endl;

    return 0;
}
